#include <avr/pgmspace.h>
#include "font.h"
#include <PString.h>

#define MATRIX_ROW 5
#define MATRIX_COL 22
#define PIN_NUMBER 11



const byte pins[PIN_NUMBER] = {2, 3, 4,5, 6, 7, 8,9,10,11,12}; //the number of the pin used for the LEDs in ordered
const byte connectionMatrix[MATRIX_ROW][MATRIX_COL][2] = { //the matrix that shows the LEDs pin connections. First Value is the Anode, second is the Catode
  {{1, 10}, {0, 10}, {1, 9}, {0, 9}, {1, 8}, {0, 8},{1, 7}, {0, 7}, {1, 6}, {0, 6}, {1, 5}, {0, 5},{1, 4}, {0, 4}, {1, 3}, {0, 3}, {1, 2}, {0, 2}, {2, 1}, {0, 1}, {2, 0}, {1, 0}}, 
  {{3, 10}, {2, 10}, {3, 9}, {2, 9}, {3, 8}, {2, 8},{3, 7}, {2, 7}, {3, 6}, {2, 6}, {3, 5}, {2, 5},{3, 4}, {2, 4}, {4, 3}, {2, 3}, {4, 2}, {3, 2}, {4, 1}, {3, 1}, {4, 0}, {3, 0}},
  {{5, 10}, {4, 10}, {5, 9}, {4, 9}, {5, 8}, {4, 8},{5, 7}, {4, 7}, {5, 6}, {4, 6}, {6, 5}, {4, 5},{6, 4}, {5, 4}, {6, 3}, {5, 3}, {6, 2}, {5, 2}, {6, 1}, {5, 1}, {6, 0}, {5, 0}},
  {{7, 10}, {6, 10}, {7, 9}, {6, 9}, {7, 8}, {6, 8},{8, 7}, {6, 7}, {8, 6}, {7, 6}, {8, 5}, {7, 5},{8, 4}, {7, 4}, {8, 3}, {7, 3}, {8, 2}, {7, 2}, {8, 1}, {7, 1}, {8, 0}, {7, 0}},
  {{9, 10}, {8, 10}, {10, 9}, {8, 9}, {10, 8},{9,8},{10, 7}, {9, 7}, {10, 6}, {9, 6}, {10, 5}, {9, 5},{10, 4}, {9, 4}, {10, 3}, {9, 3}, {10, 2}, {9, 2}, {10, 1}, {9, 1}, {10, 0}, {9, 0}}
};

bool videoram[MATRIX_ROW][MATRIX_COL] = { //the matrix that will be always used to turn ON or OFF the LEDs
  {0, 0, 0, 0, 0, 0},
  {0, 0, 0, 0, 0, 0},
  {0, 0, 0, 0, 0, 0},
  {0, 0, 0, 0, 0, 0},
  {0, 0, 0, 0, 0, 0}
};
char LEDbuffer1[15];
PString led1(LEDbuffer1, 15); // LED line 1

void multiplex() { 

  // THIS PART IS USED TO UPDATE THE CHARLIEPLEXING LEDS MATRIX
  //check from matrixState which LED to turn ON or OFF
  for (byte i = 0; i < MATRIX_ROW; i++) {
    for (byte j = 0; j < MATRIX_COL; j++) {
      if (videoram[i][j] == 1) { //turn on LED with 1 in matrixState
        pinMode(pins[connectionMatrix[i][j][0]], OUTPUT); //set positive pole to OUTPUT
        pinMode(pins[connectionMatrix[i][j][1]], OUTPUT); //set negative pole to OUTPUT
        digitalWrite(pins[connectionMatrix[i][j][0]], HIGH); //set positive pole to HIGH
        digitalWrite(pins[connectionMatrix[i][j][1]], LOW); //set negative pole to LOW
        delayMicroseconds(100); //250
        pinMode(pins[connectionMatrix[i][j][0]], INPUT); //set both positive pole and negative pole
        pinMode(pins[connectionMatrix[i][j][1]], INPUT); // to INPUT in order to turn OFF the LED
      }
      if (videoram[i][j] == 0) { //turn off LED with 0 in matrixState
        pinMode(pins[connectionMatrix[i][j][0]], INPUT); //set both positive pole and negative pole
        pinMode(pins[connectionMatrix[i][j][1]], INPUT); // to INPUT in order to turn OFF the LED
//        delayMicroseconds(200); //250
      }
    }
  }

}



void setup() {
  //configure LED pins
  for (byte i = 0; i < PIN_NUMBER; i++) {
    pinMode(pins[i], INPUT);
  }
  clearMatrix(); //clear the Matrix
   led1.begin();
   led1.print(F("Hello")); 
  display();
}


void loop() {
 for (byte m = 0; m < 200; m++) {
multiplex();
 }
  clearMatrix(); //clear the Matrix
   led1.begin();
   led1.print(millis()); 
  display();
}



void clearMatrix() {
  //clear the matrix by inserting 0 to the matrixState
  for (byte i = 0; i < MATRIX_ROW; i++) {
    for (byte j = 0; j < MATRIX_COL; j++) {
      videoram[i][j] = 0;
    }
  }
}

void fullMatrix() {
  //turn on all LEDs in the matrix by inserting 1 to the matrixState
  for (byte i = 0; i < MATRIX_ROW; i++) {
    for (byte j = 0; j < MATRIX_COL; j++) {
      videoram[i][j] = 1;
    }
  }
}

void display()
{
  int charData;
    for( int i = 0; i <10; i++){
      charData=LEDbuffer1[i];
      if (charData==0) break;
      printChar(charData, i*4);
    }
}
void printChar(int number,int idx)
{
  int colData;
  for( int i = 0; i <3; i++){
      for( int j = 0; j <5; j++){
colData = (pgm_read_byte(&font_data[((number-32 )*3)+i]));
            if ( colData & (1<<j) ) {
      videoram[j][i+idx]= 1;
    }
    else {  
      videoram[j][i+idx]= 0;
    }
    
      }
  }
}


